@@ -4,12 +4,12 @@ ENC_DEFINE("US-ASCII");
 ENC_DEFINE("Big5");
 ENC_DEFINE("Big5-HKSCS");
 ENC_DEFINE("Big5-UAO");
+ENC_DEFINE("CESU-8");
 ENC_DEFINE("CP949");
 ENC_DEFINE("Emacs-Mule");
 ENC_DEFINE("EUC-JP");
 ENC_DEFINE("EUC-KR");
 ENC_DEFINE("EUC-TW");
-ENC_DEFINE("GB2312");
 ENC_DEFINE("GB18030");
 ENC_DEFINE("GBK");
 ENC_DEFINE("ISO-8859-1");
@@ -44,6 +44,8 @@ ENC_DEFINE("Windows-1257");
 ENC_ALIAS("BINARY", "ASCII-8BIT");
 ENC_REPLICATE("IBM437", "ASCII-8BIT");
 ENC_ALIAS("CP437", "IBM437");
+ENC_REPLICATE("IBM720", "ASCII-8BIT");
+ENC_ALIAS("CP720", "IBM720");
 ENC_REPLICATE("IBM737", "ASCII-8BIT");
 ENC_ALIAS("CP737", "IBM737");
 ENC_REPLICATE("IBM775", "ASCII-8BIT");
@@ -101,6 +103,7 @@ ENC_REPLICATE("EUC-JIS-2004", "EUC-JP") /* defined at JIS X 0213:2004 */;
 ENC_ALIAS("EUC-JISX0213", "EUC-JIS-2004") /* defined at JIS X 0213:2000, and obsolete at JIS X 0213:2004 */;
 ENC_ALIAS("eucKR", "EUC-KR");
 ENC_ALIAS("eucTW", "EUC-TW");
+ENC_REPLICATE("GB2312", "EUC-KR");
 ENC_ALIAS("EUC-CN", "GB2312");
 ENC_ALIAS("eucCN", "GB2312");
 ENC_REPLICATE("GB12345", "GB2312");
@@ -169,4 +172,4 @@ ENC_REPLICATE("stateless-ISO-2022-JP-KDDI", "stateless-ISO-2022-JP");
 ENC_REPLICATE("UTF8-SoftBank", "UTF-8");
 ENC_REPLICATE("SJIS-SoftBank", "Windows-31J");
 
-#define ENCODING_COUNT 101
+#define ENCODING_COUNT 103
